@@ -20,7 +20,7 @@ from lino.core import choicelists
 from lino.core.gfks import ContentType
 from lino.modlib.extjs.ext_renderer import ExtRenderer
 
-from lino.core.actions import (ShowEmptyTable, ShowDetail, QuickLinkAction,
+from lino.core.actions import (ShowEmptyTable, ShowDetail, WrappedAction,
                                ShowInsert, ShowTable, SubmitDetail,
                                SubmitInsert, Action)
 from lino.core.boundaction import BoundAction
@@ -94,7 +94,7 @@ class Renderer(JsRenderer, JsCacheRenderer):
         self.serialise_js_code = True
         plugins = settings.SITE.plugins
         data = dict(
-            actions={a.action_name: a for a in actions},
+            actions={a.action_name: self.action2json(a) for a in actions},
             # actors={a.actor_id: a for a in self.actors_list},
             menu=settings.SITE.get_site_menu(get_user_profile()),
             suggestors=list(plugins.memo.parser.suggesters.keys()),  # [#,@] keytriggers
@@ -102,6 +102,11 @@ class Renderer(JsRenderer, JsCacheRenderer):
             site_title=settings.SITE.title,
             )
 
+        if settings.SITE.never_build_site_cache:
+            data.update(revision_number=settings.SITE.kernel.lino_version)
+        else:
+            data.update(revision_number=settings.SITE.kernel.code_mtime)
+
         if hasattr(settings.SITE, 'theme_name'):
             data.update(theme_name=settings.SITE.theme_name)
 
@@ -143,6 +148,33 @@ class Renderer(JsRenderer, JsCacheRenderer):
         self.serialise_js_code = False
         return 1
 
+    def action2json(self, v):
+        """Converts global list of all actions to json format."""
+        assert isinstance(v, Action)
+        # todo include all aux info
+        # todo include grid info
+        # todo refactor this into a all_actions object and have the bound actions ref it to reduse json size
+        result = dict(an=v.action_name,
+                      label=v.get_label(),  # todo fix this, this is a readable action, not ID for the action
+                      window_action=v.is_window_action(),
+                      http_method=v.http_method,
+                      )
+
+        if isinstance(v, WrappedAction):
+            result['an'] = v.bound_action.action.action_name
+            result['actor'] = str(v.bound_action.actor)
+
+        if v.preprocessor: result["preprocessor"] = v.preprocessor
+        # if v.combo_group: result["combo_group"] = v.combo_group
+        if v.select_rows: result['select_rows'] = v.select_rows
+        if v.submit_form_data: result['submit_form_data'] = True
+        if v.button_text: result['button_text'] = v.button_text
+
+        icon = self.get_action_icon(v)
+        if icon: result['icon'] = icon
+
+        return result
+
     def panel2json(self, p):
         assert isinstance(p, BaseLayout)
         # is a Layout instance
@@ -350,37 +382,6 @@ class Renderer(JsRenderer, JsCacheRenderer):
                     handler=js_code("function() { Lino.load_url('%s'); }" % url))
             return dict(text=v.label, href=url)
 
-        if isinstance(v, Action):
-            # todo include all aux info
-            # todo include grid info
-            # todo refactor this into a all_actions object and have the bound actions ref it to reduse json size
-            result = dict(an=v.action_name,
-                          label=v.get_label(),  # todo fix this, this is a readable action, not ID for the action
-                          window_action=v.is_window_action(),
-                          http_method=v.http_method,
-                          )
-
-            # href_target = v.get_a_href_target()
-            # if href_target:
-            #     result.update(href=href_target)
-
-            if isinstance(v, QuickLinkAction):
-                result['an'] = v.bound_action.action.action_name
-                result['actor'] = str(v.bound_action.actor)
-
-            # if v.show_in_bbar: result["bbar"] = True # not needed
-            if v.preprocessor: result["preprocessor"] = v.preprocessor
-            if v.combo_group: result["combo_group"] = v.combo_group
-            if v.select_rows: result['select_rows'] = v.select_rows
-            if v.submit_form_data: result['submit_form_data'] = True
-            if v.button_text: result['button_text'] = v.button_text
-            # if v.help_text: result['help_text'] = v.help_text
-
-            icon = self.get_action_icon(v)
-            if icon: result['icon'] = icon
-
-            return result
-
         if isinstance(v, js_code) and getattr(self, "serialise_js_code", False):
             # Convert js_code into strings so they are serialised. rather than displayed w/o quotes
             return str(v.s)
@@ -486,6 +487,8 @@ class Renderer(JsRenderer, JsCacheRenderer):
         return ('cache', file_type, filename)
 
     def actions2json(self, actions_list):  # 20210517
+        """Converts of all the boundactions of an actor to json format.
+        """
         # v is a dict of an -> BoundCction instances
         result = []
         hotkeys = []
@@ -493,35 +496,39 @@ class Renderer(JsRenderer, JsCacheRenderer):
         for ba in actions_list:
             lh = ba.get_layout_handel()
             if (lh or ba.action.window_type):
-                # and k not in [
-                #     "default_action", "insert_action", "detail_action"]:
 
                 action_descriptor = {'an': ba.action.action_name}
                 if lh:
                     action_descriptor.update(window_layout=lh.layout._formpanel_name)
                 if ba.action.window_type:
                     tbas = []
-                    combo_group = None
+                    combo_map = dict()
 
                     for a in ba.actor.get_toolbar_actions(
                         ba.action, get_user_profile()):
-                        if a.action.combo_group == combo_group\
-                            and combo_group is not None:
-                            previous = tbas.pop()
-                            if not isinstance(previous, list):
-                                previous = [previous]
-                            tba = dict(an=a.action.action_name)
-                            help_text = a.action.help_text
-                            if help_text: tba['help_text'] = help_text
-                            previous.append(tba)
-                            tbas.append(previous)
-                        else:  # is normal
-                            tba = dict(an=a.action.action_name)
-                            help_text = a.action.help_text
-                            if help_text: tba['help_text'] = help_text
-                            tbas.append(tba)
+
+                        tba = dict(an=a.action.action_name)
+                        help_text = a.action.help_text
+                        if help_text: tba['help_text'] = help_text
+                        js_handler = a.action.js_handler
+                        if js_handler:
+                            if callable(js_handler):
+                                tba['js_handler'] = js_handler(a.actor)
+                            else:
+                                tba['js_handler'] = js_handler
+
+                        if a.action.combo_group is not None:
                             combo_group = a.action.combo_group
 
+                            if combo_group in combo_map:
+                                combo_map[combo_group].append(tba)
+                            else:
+                                combo = dict(combo=combo_group)
+                                combo_map[combo_group] = [tba]
+                                tbas.append(combo)
+                        else:  # is normal
+                            tbas.append(tba)
+
                         if a.action.hotkey:
                             hotkey = vars(a.action.hotkey)
                             if not hasattr(hotkey, 'ba'):
@@ -529,6 +536,11 @@ class Renderer(JsRenderer, JsCacheRenderer):
                             if hotkey not in hotkeys:
                                 hotkeys.append(hotkey)
 
+                    if combo_map:
+                        for i, tba in enumerate(tbas):
+                            if tba.get('combo', False):
+                                tbas[i]['menu'] = combo_map[tba['combo']]
+
                     action_descriptor["toolbarActions"] = tbas
 
                 result.append(action_descriptor)
