@@ -47,7 +47,7 @@ from lino.core.store import get_atomizer
 
 from lino.modlib.users.utils import get_user_profile, with_user_profile
 from lino.api import rt
-from lino.modlib.extjs.views import ApiElement, elem2rec_empty
+from lino.modlib.extjs.views import ApiElement, test_version_mismatch, elem2rec_empty
 
 
 def find(itter, target, key=None):
@@ -142,6 +142,7 @@ class ApiList(View):
             if isinstance(ar.bound_action.action, ShowEmptyTable):
                 elem = ar.create_instance()
                 datarec = elem2rec_empty(ar, ar.ah, elem)
+                datarec.update(test_version_mismatch(request))
                 return json_response(datarec)
 
             display_mode = request.GET.get(
@@ -188,6 +189,7 @@ class ApiList(View):
                 kw.update(
                     param_values=ar.actor.params_layout.params_store.pv2dict(
                         ar, ar.param_values))
+            kw.update(test_version_mismatch(request))
             return json_response(kw)
 
         return settings.SITE.kernel.run_action(ar)
